@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -75,6 +72,7 @@ bool IOFireWireDeviceAux::init( IOFireWireDevice * primary )
 	{
 		fTerminated = false;
 		fMaxSpeed = kFWSpeedMaximum;
+		fUnitCount = 0;
 	}
 	
 	return success;
@@ -113,6 +111,24 @@ void IOFireWireDeviceAux::setMaxSpeed( IOFWSpeed speed )
 	fPrimary->fControl->openGate();
 }
 
+// setUnitCount
+//
+//
+
+void IOFireWireDeviceAux::setUnitCount( UInt32 count )
+{
+	fUnitCount = count;
+}
+
+// getUnitCount
+//
+//
+
+UInt32 IOFireWireDeviceAux::getUnitCount( void )
+{
+	return fUnitCount;
+}
+
 // free
 //
 //
@@ -1206,6 +1222,9 @@ IOReturn IOFireWireDevice::processUnitDirectories( OSSet * unitSet )
 	
 	if( status == kIOReturnSuccess )
 	{	
+		UInt32 count = unitSet->getCount();
+		setUnitCount( count );
+		
 		iterator = OSCollectionIterator::withCollection( unitSet );
 		iterator->reset();
 		
