@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -32,6 +29,12 @@
  */
 /*
 	$Log: IOFireWireLibPriv.h,v $
+	Revision 1.33  2004/01/22 01:50:01  niels
+	fix user space physical address space getPhysicalSegments
+	
+	Revision 1.32  2003/12/19 22:07:46  niels
+	send force stop when channel dies/system sleeps
+	
 	Revision 1.31  2003/11/07 21:01:19  niels
 	*** empty log message ***
 	
@@ -256,12 +259,12 @@ namespace IOFireWireLib {
 		UInt32		addressLo ;
 	}  ;
 	
-	struct PhysicalAddressSpaceCreateParams
-	{
-		UInt32		size ;
-		void*		backingStore ;
-		UInt32		flags ;
-	}  ;
+//	struct PhysicalAddressSpaceCreateParams
+//	{
+//		UInt32		size ;
+//		void*		backingStore ;
+//		UInt32		flags ;
+//	}  ;
 	
 	// 
 	// config ROM
@@ -385,6 +388,7 @@ namespace IOFireWireLib {
 		kLocalIsochPort_Allocate,
 		kLocalIsochPort_ModifyJumpDCL_d,
 		kLocalIsochPort_Notify_d,
+		kLocalIsochPort_SetChannel,
 //		kLocalIsochPort_ModifyTransferPacketDCLSize,
 //		kLocalIsochPort_ModifyTransferPacketDCL,
 		
